from .agem.agem import AGEM
from .coral.coral import DeepCORAL
from .erm.erm import ERM
from .ewc.ewc import EWC
from .ft.ft import FT
from .groupdro.groupdro import GroupDRO
from .irm.irm import IRM
from .si.si import SI
