configs_fmow_coral = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'coral', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.99, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 0.9, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_groupdro = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'groupdro', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_irm = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'irm', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_erm = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'erm', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_erm_lisa = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'erm', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': True, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_erm_mixup = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'erm', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': True, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_agem = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'agem', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 1000, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_ewc = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'ewc', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 0.5, 'gamma': 1.0, 'online': True, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_ft = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'ft', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_si = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'si', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_simclr = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'simclr', 'device': 0, 'random_seed': 1, 'train_update_iter': 250, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 250, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_swa = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'swa', 'device': 0, 'random_seed': 1, 'train_update_iter': 500, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 0, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
configs_fmow_swav = {'dataset': 'fmow', 'regression': False, 'prediction_type': None, 'method': 'swav', 'device': 0, 'random_seed': 1, 'train_update_iter': 250, 'lr': 0.0001, 'momentum': 0.9, 'weight_decay': 0.0, 'mini_batch_size': 64, 'reduced_train_prop': None, 'eval_fix': False, 'difficulty': False, 'split_time': 13, 'eval_next_timestamps': 6, 'load_model': False, 'eval_all_timestamps': False, 'K': 1, 'lisa': False, 'lisa_intra_domain': False, 'mixup': False, 'lisa_start_time': 0, 'mix_alpha': 2.0, 'cut_mix': False, 'num_groups': 3, 'group_size': 3, 'non_overlapping': False, 'ewc_lambda': 1.0, 'gamma': 1.0, 'online': False, 'fisher_n': None, 'emp_FI': False, 'buffer_size': 100, 'coral_lambda': 1.0, 'irm_lambda': 1.0, 'irm_penalty_anneal_iters': 0, 'si_c': 0.1, 'epsilon': 0.001, 'ssl_finetune_iter': 250, 'data_dir': '../Data', 'log_dir': './checkpoints', 'results_dir': './results', 'num_workers': 0}
